/*
  RPCEmu - An Acorn system emulator

  Copyright (C) 2021 RiscOS Cloverleaf

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* HOST<->GUEST clibpoard integration support */

#include <stdio.h>
#include <string.h>
#include "hostclipboard.h"
#include "utf8riscos.h"
#include "mem.h"
#include "rpcemu.h"
#include "network.h"

uint32_t cliptask_pollword_addr = 0;
int alphabet = 100;
char* host_clipboard = NULL;

void clipboard_swi(uint32_t r0, uint32_t r1, uint32_t r2, uint32_t r3, uint32_t r4, uint32_t r5, uint32_t *retr0, uint32_t *retr1) {
//    fprintf(stderr, "clipboard_swi: r0:%d r1:%x r2:%d\n",
//           r0, r1, r2);
    switch (r0) {
        case ARCEM_SWI_CLIPBOARD_SETUP:
            cliptask_pollword_addr = r1;
            alphabet = r2;
            break;
        case ARCEM_SWI_CLIPBOARD_HOST_LEN: {
            if (host_clipboard != NULL) {
                *retr0 = strlen(host_clipboard);
            } else {
                *retr0 = 0;
            }
//            fprintf(stderr, "ARCEM_SWI_CLIPBOARD_HOST_LEN %d\n", *retr0);
            break;
        }
        case ARCEM_SWI_CLIPBOARD_HOST_GET: {
            // r1 dest addr
            // r2 dest len
            size_t cliplen = strlen(host_clipboard);
            if (r2 < cliplen + 1) {
                cliplen = r2;
            }
//            fprintf(stderr, "ARCEM_SWI_CLIPBOARD_HOST_GET %d - %s\n", cliplen, host_clipboard);
            memcpyfromhost(r1, host_clipboard, cliplen);
            mem_write8(r1 + cliplen, 0);
//            fprintf(stderr, "ARCEM_SWI_CLIPBOARD_HOST_GET %s\n", host_clipboard);
            break;
        }
        case ARCEM_SWI_CLIPBOARD_HOST_SET: {
            // r1 src addr
            // r2 src len
            if (host_clipboard) {
                free(host_clipboard);
            }
            host_clipboard = malloc(r2+1);
            memcpytohost(host_clipboard, r1, r2);
            host_clipboard[r2] = 0;
//            fprintf(stderr, "ARCEM_SWI_CLIPBOARD_HOST_SET %x %d %s\n", r1, r2, host_clipboard);
            char *utf8clipboard;
            utf8_from_local_encoding(host_clipboard, alphabet, &utf8clipboard);
            if (!utf8clipboard) {
#if defined __linux || defined __linux__
                fprintf(stderr, "clipboard utf8_from_local_encoding() failed\n");
#endif
                rpclog("clipboard utf8_from_local_encoding() failed\n");
            } else {
//#if defined __linux || defined __linux__
//                fprintf(stderr, "set_host_clipboard_data: '%s'\n", utf8clipboard);
//#endif
//                rpclog("set_host_clipboard_data: '%s'\n", utf8clipboard);
                rpcemu_set_host_clipboard(utf8clipboard);
                free(utf8clipboard);
            }
            break;
        }
        default:
            break;
    }
}

void clipboard_changed_on_host_notify(char *txt) {
//    fprintf(stderr, "clipboard_changed_on_host pollword:%x\n", cliptask_pollword_addr);
    char *new_host_clipboard, *old;

    if (!cliptask_pollword_addr) {
        return;
    }

    utf8_to_local_encoding(txt, alphabet, &new_host_clipboard);
    if (!new_host_clipboard) {
#if defined __linux || defined __linux__
        fprintf(stderr, "clipboard utf8_to_local_encoding() failed\n");
#endif
        rpclog("clipboard utf8_to_local_encoding() failed\n");
        return;
    }

    if (host_clipboard != NULL) {
        if (strcmp(new_host_clipboard, host_clipboard) == 0) {
//            fprintf(stderr, "clipboard_changed_on_host clipboard the same\n");
            free(new_host_clipboard);
            return;
        }
    }

    old = host_clipboard;
    host_clipboard = new_host_clipboard;
    free(old);

    mem_write32(cliptask_pollword_addr, ARCEM_POLLWORD_HOST_CLIPBORARD_CHANGED);
}


